#!/usr/bin/env python

"""
Dictionary merging classes.

Copyright (C) 2009 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from itermerge import itermerge

class Merger:

    "Merge files."

    def __init__(self, writer, readers):
        self.writer = writer
        self.readers = readers

    def close(self):
        for reader in self.readers:
            reader.close()
        self.writer.close()

class TermDictionaryMerger(Merger):

    "Merge term and position files."

    def merge(self):

        """
        Merge terms and positions from the readers, sending them to the writer.
        """

        last_term = None
        current_readers = []

        for term, frequency, doc_frequency, positions in itermerge(self.readers):
            if term == last_term:
                current_readers.append(positions)
            else:
                if current_readers:
                    self.writer.write_term_positions(last_term, itermerge(current_readers))
                last_term = term
                current_readers = [positions]
        else:
            if current_readers:
                self.writer.write_term_positions(last_term, itermerge(current_readers))

class FieldDictionaryMerger(Merger):

    "Merge field files."

    def merge(self):

        """
        Merge fields from the readers, sending them to the writer.
        """

        for docnum, fields in itermerge(self.readers):
            self.writer.write_fields(docnum, fields)

# vim: tabstop=4 expandtab shiftwidth=4
